@@ -25,11 +25,12 @@ import (
 	"testing"
 
 	. "github.com/onsi/gomega"
-	apiv1 "github.com/stefanprodan/timoni/api/v1alpha1"
 	corev1 "k8s.io/api/core/v1"
 	apierrors "k8s.io/apimachinery/pkg/api/errors"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"sigs.k8s.io/controller-runtime/pkg/client"
+
+	apiv1 "github.com/stefanprodan/timoni/api/v1alpha1"
 )
 
 func TestApply(t *testing.T) {
@@ -41,7 +42,7 @@ func TestApply(t *testing.T) {
 	t.Run("creates instance with default values", func(t *testing.T) {
 		g := NewWithT(t)
 		output, err := executeCommand(fmt.Sprintf(
-			"apply -n %s %s %s -p main --wait",
+			"apply -n %s %s %s -p main --wait --timeout=10s",
 			namespace,
 			name,
 			modPath,
@@ -61,6 +62,8 @@ func TestApply(t *testing.T) {
 
 		g.Expect(clientCM.GetLabels()).To(HaveKeyWithValue(tGroup+"/name", name))
 		g.Expect(clientCM.GetLabels()).To(HaveKeyWithValue(tGroup+"/namespace", namespace))
+		g.Expect(clientCM.GetLabels()).To(HaveKeyWithValue("app.kubernetes.io/version", "0.0.0-devel"))
+		g.Expect(clientCM.GetLabels()).To(HaveKey("app.kubernetes.io/kube"))
 	})
 
 	t.Run("updates instance with custom values", func(t *testing.T) {
@@ -313,3 +316,41 @@ func TestApply_Actions(t *testing.T) {
 		g.Expect(err).ToNot(HaveOccurred())
 	})
 }
+
+func TestApply_GlobalResources(t *testing.T) {
+	modPath := "testdata/module"
+	name := rnd("my-instance", 5)
+	namespace := rnd("my-namespace", 5)
+	nsObj := &corev1.Namespace{
+		ObjectMeta: metav1.ObjectMeta{
+			Name: fmt.Sprintf("%s-ns", name),
+		},
+	}
+
+	t.Run("creates instance with global objects", func(t *testing.T) {
+		g := NewWithT(t)
+		output, err := executeCommandWithIn(fmt.Sprintf(
+			"apply -n %s %s %s -f- -p main --wait --timeout=10s",
+			namespace,
+			name,
+			modPath,
+		), strings.NewReader("values: ns: enabled: true"))
+		g.Expect(err).ToNot(HaveOccurred())
+		t.Log("\n", output)
+
+		ns := nsObj.DeepCopy()
+		err = envTestClient.Get(context.Background(), client.ObjectKeyFromObject(ns), ns)
+		g.Expect(err).ToNot(HaveOccurred())
+	})
+
+	t.Run("uninstalls instance", func(t *testing.T) {
+		g := NewWithT(t)
+		output, err := executeCommand(fmt.Sprintf(
+			"delete -n %s %s --wait=false",
+			namespace,
+			name,
+		))
+		g.Expect(err).ToNot(HaveOccurred())
+		t.Log("\n", output)
+	})
+}
