@@ -18,6 +18,7 @@ package main
 
 import (
 	"context"
+	"errors"
 	"fmt"
 	"io"
 	"os"
@@ -33,8 +34,11 @@ import (
 var initModCmd = &cobra.Command{
 	Use:   "init [MODULE NAME] [PATH]",
 	Short: "Create a module along with common files and directories",
-	Example: `  # create a module in the current directory
-  timoni mod init my-app .
+	Example: `  # Create a module in the current directory
+  timoni mod init my-app
+
+  # Create a module at the specified path
+  timoni mod init my-app ./modules
 `,
 	RunE: runInitModCmd,
 }
@@ -56,12 +60,16 @@ const (
 )
 
 func runInitModCmd(cmd *cobra.Command, args []string) error {
-	if len(args) < 2 {
-		return fmt.Errorf("module name and path are required")
+	if len(args) < 1 {
+		return errors.New("module name is required")
 	}
-
 	initModArgs.name = args[0]
-	initModArgs.path = args[1]
+
+	if len(args) == 2 {
+		initModArgs.path = args[1]
+	} else {
+		initModArgs.path = "."
+	}
 
 	log := LoggerFrom(cmd.Context())
 
@@ -163,7 +171,7 @@ func initModuleFromTemplate(mName, mTmpl, src string, dst string) (err error) {
 		return err
 	}
 	if !si.IsDir() {
-		return fmt.Errorf("source is not a directory")
+		return errors.New("source is not a directory")
 	}
 
 	_, err = os.Stat(dst)
